@@ -317,7 +317,7 @@ def memorize_project(cfg):
         cfg.pro_stat = cfg.start_pro_stat
     else:
         cfg.pro_stat = cfg.start_kicad_pro_stat
-    with open(cfg.pro_name) as f:
+    with open(cfg.pro_name, 'rb') as f:
         cfg.pro_content = f.read()
     atexit.register(restore_project, cfg)
     if cfg.prl_ext:
@@ -325,7 +325,7 @@ def memorize_project(cfg):
         if not os.path.isfile(cfg.prl_name):
             return
         cfg.prl_stat = cfg.start_kicad_prl_stat
-        with open(cfg.prl_name) as f:
+        with open(cfg.prl_name, 'rb') as f:
             cfg.prl_content = f.read()
 
 
@@ -341,7 +341,7 @@ def _restore_project(name, stat_v, content):
         if not pro_found or new_stat.st_mtime != stat_v.st_mtime:
             logger.debug('Restoring the project file')
             os.rename(name, name+'-bak')
-            with open(name, 'wt') as f:
+            with open(name, 'wb') as f:
                 f.write(content)
             os.utime(name, times=(stat_v.st_atime, stat_v.st_mtime))
 
@@ -355,12 +355,14 @@ def restore_project(cfg):
 
 def get_log_files(out_dir, app_name, also_interposer=False):
     if log.get_level() > 2:
+        os.makedirs(out_dir, exist_ok=True)
         flog_out = open(os.path.join(out_dir, app_name+'_out.log'), 'wt')
         flog_err = open(os.path.join(out_dir, app_name+'_err.log'), 'wt')
         logger.debug('Redirecting '+app_name+' output to '+app_name+'*.log')
     else:
         flog_out = flog_err = DEVNULL
     if also_interposer:
+        os.makedirs(out_dir, exist_ok=True)
         fname = os.path.join(out_dir, app_name+'_interposer.log')
         flog_int = open(fname, 'wt')
         logger.debug('Saving '+app_name+' interposer dialog to '+fname)
