@@ -4,7 +4,6 @@
 # License: Apache 2.0
 # Project: KiAuto (formerly kicad-automation-scripts)
 import atexit
-from importlib.machinery import EXTENSION_SUFFIXES
 import os
 import platform
 import psutil
@@ -23,15 +22,14 @@ KICAD_EXIT_MSG = '>>exit<<'
 INTERPOSER_OPS = 'interposer_options.txt'
 IGNORED_DIALOG_MSGS = {'The quick brown fox jumps over the lazy dog.', '0123456789'}
 BOGUS_FILENAME = '#'
+# These dialogs are asynchronous, they can pop-up at anytime.
+# One example is when the .kicad_wks is missing, KiCad starts drawing and then detects it.
+INFO_DIALOGS = {'KiCad PCB Editor Information', 'KiCad Schematic Editor Information'}
 
 
 def check_interposer(args, logger, cfg):
     # Name of the interposer library
-    # Why Debian thinks the interposer is a Python extension?
-    for ext in EXTENSION_SUFFIXES:
-        interposer_lib = os.path.abspath(os.path.join(os.path.dirname(__file__), 'interposer', 'libinterposer'+ext))
-        if os.path.isfile(interposer_lib):
-            break
+    interposer_lib = os.path.abspath(os.path.join(os.path.dirname(__file__), 'interposer', 'libinterposer.so'))
     if (not os.path.isfile(interposer_lib) or  # The lib isn't there
        args.disable_interposer or              # The user disabled it
        os.environ.get('KIAUTO_INTERPOSER_DISABLE') or  # The user disabled it using the environment
@@ -190,7 +188,15 @@ def wait_queue(cfg, strs='', starts=False, times=1, timeout=300, do_to=True, kic
            not(not cfg.ki5 and line.endswith(cfg.window_title_end))):
             # We aren't expecting a window, but something seems to be there
             # Note that window title change is normal when we expect KiCad exiting
-            unknown_dialog(cfg, line[17:])
+            title = line[17:]
+            if title in INFO_DIALOGS:
+                # Async dialogs
+                dismiss_pcb_info(cfg, title)
+            elif title == 'pcbnew Warning':
+                # KiCad 5 error during post-load, before releasing the CPU
+                dismiss_pcbnew_warning(cfg, title)
+            else:
+                unknown_dialog(cfg, title)
     if do_to:
         raise RuntimeError('Timed out waiting for `{}`'.format(strs))
 
@@ -244,10 +250,25 @@ def open_dialog_i(cfg, name, keys, no_show=False, no_wait=False, no_main=False,
     if isinstance(keys, str):
         keys = ['key', keys]
     xdotool(keys)
-    pre_gtk = 'GTK:Window Title:' if no_show else 'GTK:Window Show:'
+    pre_gtk_title = 'GTK:Window Title:'
+    pre_gtk = pre_gtk_title if no_show else 'GTK:Window Show:'
     if isinstance(name, str):
         name = [name]
-    res = wait_queue(cfg, [pre_gtk+f for f in name], with_windows=True)
+    name_w_pre = [pre_gtk+f for f in name]
+    # Add the async dialogs
+    for t in INFO_DIALOGS:
+        name_w_pre.append(pre_gtk_title+t)
+    # Wait for our dialog or any async dialog
+    # Note: wait_queue won't dismiss them because we use "with_windows=True"
+    while True:
+        res = wait_queue(cfg, name_w_pre, with_windows=True)
+        title = res[len(pre_gtk_title):]
+        if title not in INFO_DIALOGS:
+            break
+        # Get rid of the info dialog
+        dismiss_pcb_info(cfg, title)
+        # Send the keys again
+        xdotool(keys)
     name = res[len(pre_gtk):]
     if no_wait:
         return name, None
@@ -316,6 +337,9 @@ def send_keys(cfg, msg, keys, closes=None, delay_io=False):
 
 
 def wait_create_i(cfg, name, fn=None):
+    """ Wait for open+close of the file.
+        Also look for them in the collected_io messages.
+        And if we just get close forget about the open. """
     cfg.logger.info('Wait for '+name+' file creation')
     wait_point(cfg)
     if fn is None:
@@ -329,10 +353,12 @@ def wait_create_i(cfg, name, fn=None):
     else:
         got_open = False
         got_close = False
-    if got_open:
-        cfg.logger.debug('Found IO '+open_msg)
+    if got_open or got_close:
+        if got_open:
+            cfg.logger.debug('Found IO '+open_msg)
     else:
-        wait_queue(cfg, open_msg)
+        msg = wait_queue(cfg, [open_msg, close_msg], starts=True)
+        got_close = msg.startswith(close_msg)
     if got_close:
         cfg.logger.debug('Found IO '+close_msg)
     else:
@@ -356,12 +382,21 @@ def collect_dialog_messages(cfg, title):
     return msgs
 
 
-def unknown_dialog(cfg, title, msgs=None):
+def exit_pcb_ees_error(cfg):
+    exit(PCBNEW_ERROR if cfg.is_pcbnew else EESCHEMA_ERROR)
+
+
+def unknown_dialog(cfg, title, msgs=None, fatal=True):
     if msgs is None:
         msgs = collect_dialog_messages(cfg, title)
-    cfg.logger.error('Unknown KiCad dialog: '+title)
-    cfg.logger.error('Potential dialog messages: '+str(msgs))
-    exit(PCBNEW_ERROR if cfg.is_pcbnew else EESCHEMA_ERROR)
+    msg_unk = 'Unknown KiCad dialog: '+title
+    msg_msgs = 'Potential dialog messages: '+str(msgs)
+    if fatal:
+        cfg.logger.error(msg_unk)
+        cfg.logger.error(msg_msgs)
+        exit_pcb_ees_error(cfg)
+    cfg.logger.warning(msg_unk)
+    cfg.logger.warning(msg_msgs)
 
 
 def dismiss_dialog(cfg, title, keys):
@@ -393,6 +428,10 @@ def dismiss_error(cfg, title):
         cfg.logger.warning('Missing libraries, please fix it')
         dismiss_dialog(cfg, title, 'Return')
         return
+    if 'The entire schematic could not be loaded.  Errors occurred attempting to load hierarchical sheets.' in msgs:
+        # KiCad 6 loading a sheet, but sub-sheets are missing
+        cfg.logger.error('Error loading schematic file. Missing schematic files?')
+        exit(EESCHEMA_ERROR)
     unknown_dialog(cfg, title, msgs)
 
 
@@ -426,13 +465,26 @@ def dismiss_warning(cfg, title):
     kind = 'PCB' if cfg.is_pcbnew else 'Schematic'
     if kind+' file "'+cfg.input_file+'" is already open.' in msgs:
         cfg.logger.error('File already opened by another KiCad instance')
-        exit(PCBNEW_ERROR if cfg.is_pcbnew else EESCHEMA_ERROR)
+        exit_pcb_ees_error(cfg)
     if 'Error loading schematic file "'+os.path.abspath(cfg.input_file)+'".' in msgs:
         cfg.logger.error('eeschema reported an error while loading the schematic')
         exit(EESCHEMA_ERROR)
     unknown_dialog(cfg, title, msgs)
 
 
+def dismiss_pcbnew_warning(cfg, title):
+    """ Pad in invalid layer """
+    msgs = collect_dialog_messages(cfg, title)
+    # More generic cases
+    for msg in msgs:
+        # Warning about pad using an invalid layer
+        if msg.endswith("could not find valid layer for pad"):
+            cfg.logger.warning(msg)
+            dismiss_dialog(cfg, title, 'Return')
+            return
+    unknown_dialog(cfg, title, msgs)
+
+
 def dismiss_remap_symbols(cfg, title):
     """ KiCad 5 opening an old file """
     msgs = collect_dialog_messages(cfg, title)
@@ -451,7 +503,22 @@ def dismiss_save_changes(cfg, title):
         dismiss_dialog(cfg, title, ['Left', 'Left', 'Return'])
         return
     cfg.logger.error('Save dialog without correct messages')
-    exit(PCBNEW_ERROR if cfg.is_pcbnew else EESCHEMA_ERROR)
+    exit_pcb_ees_error(cfg)
+
+
+def dismiss_pcb_info(cfg, title):
+    """ KiCad 6 information, we know about missing worksheet style """
+    msgs = collect_dialog_messages(cfg, title)
+    found = False
+    for msg in msgs:
+        if msg.startswith("Drawing sheet ") and msg.endswith(" not found."):
+            cfg.logger.warning("Missing worksheet file (.kicad_wks)")
+            cfg.logger.warning(msg)
+            found = True
+            break
+    if not found:
+        unknown_dialog(cfg, title, msgs, fatal=False)
+    dismiss_dialog(cfg, title, 'Return')
 
 
 def exit_kicad_i(cfg):
@@ -594,7 +661,11 @@ def wait_start_by_msg(cfg):
             dismiss_already_running(cfg, title)
         elif title == 'Warning':
             dismiss_warning(cfg, title)
+        elif title == 'pcbnew Warning':
+            dismiss_pcbnew_warning(cfg, title)
         elif title == 'Remap Symbols':
             dismiss_remap_symbols(cfg, title)
+        elif title in INFO_DIALOGS:
+            dismiss_pcb_info(cfg, title)
         else:
             unknown_dialog(cfg, title)
